#!/bin/sh -e
# SPDX-License-Identifier: MIT

BRIDGE_NAME="$1"
BRIDGE_ADDR="$2"

HW_MODE="$3"
CHANNEL="$4"
SSID="$5"
WPA_PASSPHRASE="$6"
INTERFACE="$7"

BRIDGE_CON_FILE="/etc/NetworkManager/system-connections/$BRIDGE_NAME.nmconnection"

HOSTAPD_CONF="/etc/hostapd/hostapd.conf"
BRIDGE_IF="br_ap"

trap catch ERR

catch() {
	if nmcli con show "$BRIDGE_NAME" >/dev/null 2>&1; then
		nmcli con del "$BRIDGE_NAME"
		persist_file -d "$BRIDGE_CON_FILE" || :
	fi


	if [ -e /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf ]; then
		persist_file -d /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf
	fi

	if [ -n "$NMCLI_IGNORE_IF" ]; then
		nmcli d set "$NMCLI_IGNORE_IF" managed yes
		nmcli d wifi list --rescan yes
	fi

	if [ -e "/etc/runlevels/default/hostapd" ]; then
		rc-update del hostapd
		persist_file -d /etc/runlevels/default/hostapd "$HOSTAPD_CONF" || :
	fi

	echo "Could not set up Access point." >&2
}

if ! nmcli con show "$BRIDGE_NAME" >/dev/null 2>&1; then
	nmcli con add con-name "$BRIDGE_NAME" type bridge ifname "$BRIDGE_IF"
	nmcli con mod "$BRIDGE_NAME" ipv4.method manual ipv4.address "$BRIDGE_ADDR" bridge.stp off
else
	nmcli con mod "$BRIDGE_NAME" ifname "$BRIDGE_IF" ipv4.method manual ipv4.address "$BRIDGE_ADDR" bridge.stp off
fi
nmcli con up "$BRIDGE_NAME"
persist_file "$BRIDGE_CON_FILE"

# disable nmcli side
NMCLI_IGNORE_IF="$INTERFACE"
if [ "$INTERFACE" = uap0 ] && [ -e /sys/class/net/mlan0 ]; then
	NMCLI_IGNORE_IF=mlan0
fi
cat > /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf <<EOF
[device_abosweb_$NMCLI_IGNORE_IF]
match-device=interface-name:$NMCLI_IGNORE_IF
managed=0
EOF
persist_file /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf
nmcli d set "$NMCLI_IGNORE_IF" managed no
ip addr flush dev "$NMCLI_IGNORE_IF"
if [ "$INTERFACE" != "$NMCLI_IGNORE_IF" ]; then
	ip link set down dev "$NMCLI_IGNORE_IF"
fi

ip link set up dev "$INTERFACE"

sed -e "s/^hw_mode=.*/hw_mode=$HW_MODE/" \
	-e "s/^channel=.*/channel=$CHANNEL/" \
	-e "s/^ssid=.*/ssid=$SSID/" \
	-e "s/^wpa_passphrase=.*/wpa_passphrase=${WPA_PASSPHRASE//\//\\/}/" \
	-e "s/^interface=.*/interface=$INTERFACE/" \
	-e "s/^bridge=.*/bridge=$BRIDGE_IF/" \
	"$HOSTAPD_CONF".example > "$HOSTAPD_CONF"

persist_file "$HOSTAPD_CONF"
rc-service hostapd restart
rc-update add hostapd
persist_file /etc/runlevels/default/hostapd
